import React from "react";
import { DOMEventListener, KEYS, mergeClasses } from "../../shared";
import { MonkeyPatchDropdown } from "./monkey-patch-dropdown";
import { PureComponent, cloneElement, createRef } from "react";
import { Ref } from "semantic-ui-react";
import { TagsPickerDropdownMenu } from "./tags-picker-dropdown-menu";
import { TagsPickerDropdownSearchInput } from "./tags-picker-dropdown-search-input";
import { TagsPickerDropdownTrigger } from "./tags-picker-dropdown-trigger";
import { arrayOf, bool, element, func, number, object, oneOf, oneOfType, shape, string } from "prop-types";
import { debounce, isFunction, isNil } from "lodash";
import { isElement } from "react-is"; // Sizes constants are duplicated here until https://github.com/reactjs/react-docgen/pull/352 is merged. Otherwise it will not render properly in the docs.

const SIZES = ["small", "medium", "large"]; // Duplicated here until https://github.com/reactjs/react-docgen/pull/352 is merged. Otherwise the props will not render properly in the docs.

const ITEM_SHAPE = {
  text: string.isRequired,
  value: string.isRequired
};
export class TagsPickerDropdown extends PureComponent {
  constructor(...args) {
    super(...args);
    this.state = {
      keyboardItem: null,
      keyboardIndex: null
    };
    this._triggerRef = /*#__PURE__*/createRef();
    this._dropdownRef = /*#__PURE__*/createRef();
    this._hasFocus = false;

    this.handleDocumentKeyDown = event => {
      switch (event.keyCode) {
        case KEYS.esc:
          this.handleDocumentEscape(event);
          break;

        case KEYS.enter:
          this.handleDocumentEnter(event);
          break;

        case KEYS.up:
          this.handleDocumentUp(event);
          break;

        case KEYS.down:
          this.handleDocumentDown(event);
          break;
      }
    };

    this.handleDocumentEscape = event => {
      this.close(event);
    };

    this.handleDocumentEnter = event => {
      const {
        keyboardItem
      } = this.state;

      if (!isNil(keyboardItem)) {
        this.selectItem(event, keyboardItem);
      }
    };

    this.handleDocumentUp = () => {
      const {
        items
      } = this.props;
      const {
        keyboardIndex
      } = this.state;

      if (items.length > 0) {
        if (!isNil(keyboardIndex)) {
          if (keyboardIndex > 0) {
            const newKeyboardIndex = keyboardIndex - 1;
            this.setKeyboardItem(items[newKeyboardIndex], newKeyboardIndex);
          }
        }
      }
    };

    this.handleDocumentDown = () => {
      const {
        items
      } = this.props;
      const {
        keyboardIndex
      } = this.state;

      if (items.length > 0) {
        if (isNil(keyboardIndex)) {
          this.setKeyboardItem(items[0], 0);
        } else {
          if (keyboardIndex < items.length - 1) {
            const newKeyboardIndex = keyboardIndex + 1;
            this.setKeyboardItem(items[newKeyboardIndex], newKeyboardIndex);
          }
        }
      }
    };

    this.handleDropdownFocusIn = () => {
      this._hasFocus = true;
    };

    this.handleDropdownFocusOut = event => {
      this._hasFocus = false; // The check is delayed because between leaving the old element and entering the new element the active element will always be the document/body itself.

      setTimeout(() => {
        if (!this._hasFocus) {
          this.close(event);
        }
      }, 0);
    };

    this.handleDocumentClick = event => {
      if (this._dropdownRef.current) {
        if (!this._dropdownRef.current.contains(event.target)) {
          this.close(event);
        }
      }
    };

    this.handleTriggerOpen = event => {
      this.open(event);
    };

    this.handleTriggerClose = event => {
      this.close(event);
    };

    this.handleSearchChange = (event, query) => {
      this.onSearch(event, query);
    };

    this.handleItemClick = (event, item) => {
      this.selectItem(event, item);
    };

    this.onSearch = this.props.debounceDelay !== 0 ? debounce(this.props.onSearch, this.props.debounceDelay) : this.props.onSearch;

    this.renderTrigger = () => {
      const {
        trigger,
        triggerText,
        open,
        disabled,
        triggerSize,
        active,
        focus,
        hover
      } = this.props;
      return /*#__PURE__*/cloneElement(trigger, {
        onOpen: this.handleTriggerOpen,
        onClose: this.handleTriggerClose,
        text: triggerText,
        open,
        disabled,
        size: triggerSize,
        active,
        focus,
        hover,
        ref: this._triggerRef
      });
    };

    this.renderSearchInput = () => {
      const {
        searchInput,
        placeholder
      } = this.props;
      const props = {
        onChange: this.handleSearchChange,
        placeholder
      };

      if (isElement(searchInput)) {
        return /*#__PURE__*/cloneElement(searchInput, props);
      }

      return /*#__PURE__*/React.createElement(TagsPickerDropdownSearchInput, Object.assign({}, props, searchInput));
    };

    this.renderMenu = () => {
      const {
        menu,
        items,
        noResultsMessage
      } = this.props;
      const {
        keyboardItem
      } = this.state;
      return /*#__PURE__*/cloneElement(menu, {
        items: items,
        onItemClick: this.handleItemClick,
        searchInput: this.renderSearchInput(),
        noResultsMessage,
        keyboardItem
      });
    };
  }

  componentDidUpdate(prevProps) {
    const {
      items
    } = this.props;

    if (prevProps.items !== items) {
      this.setKeyboardItem(null, null);
    }
  }

  componentWillUnmount() {
    this.cancelOnSearchDebounce();
  }

  cancelOnSearchDebounce() {
    if (isFunction(this.onSearch.cancel)) {
      this.onSearch.cancel();
    }
  }

  open(event) {
    const {
      onOpen
    } = this.props;
    this.setKeyboardItem(null, null);
    onOpen(event);
  }

  close(event) {
    const {
      onClose
    } = this.props;
    setTimeout(() => {
      if (!isNil(this._triggerRef.current)) {
        this._triggerRef.current.focus();
      }
    }, 0);
    onClose(event);
  }

  setKeyboardItem(item, index) {
    this.setState({
      keyboardItem: item,
      keyboardIndex: index
    });
  }

  selectItem(event, item) {
    const {
      items,
      onItemSelect,
      closeOnSelect
    } = this.props;

    if (closeOnSelect || items.length === 1) {
      this.close(event);
    } // Defering onItemSelect ensure that document click has already been handled and the dropdown is not closed because the item is not part of the menu anymore.


    setTimeout(() => {
      const selectedItem = items.find(x => x.value === item.value);
      onItemSelect(event, selectedItem);
    }, 0);
  }

  render() {
    const {
      open,
      disabled,
      className
    } = this.props;
    const classes = mergeClasses("no-icons", className);
    return /*#__PURE__*/React.createElement(React.Fragment, null, /*#__PURE__*/React.createElement(Ref, {
      innerRef: this._dropdownRef
    }, /*#__PURE__*/React.createElement(MonkeyPatchDropdown, {
      open: open,
      trigger: this.renderTrigger(),
      disabled: disabled // Otherwise the "listbox" div will be focus first instead of the trigger button.
      ,
      tabIndex: "-1",
      upward: false,
      floating: true,
      className: classes
    }, open ? this.renderMenu() : null)), open ? [/*#__PURE__*/React.createElement(DOMEventListener, {
      name: "keydown",
      on: this.handleDocumentKeyDown,
      key: "0"
    }), /*#__PURE__*/React.createElement(DOMEventListener, {
      name: "click",
      on: this.handleDocumentClick,
      key: "1"
    }), /*#__PURE__*/React.createElement(DOMEventListener, {
      target: this._dropdownRef,
      name: "focusin",
      on: this.handleDropdownFocusIn,
      key: "2"
    }), /*#__PURE__*/React.createElement(DOMEventListener, {
      target: this._dropdownRef,
      name: "focusout",
      on: this.handleDropdownFocusOut,
      key: "3"
    })] : null);
  }

}
TagsPickerDropdown.displayName = "TagsPickerDropdown";
TagsPickerDropdown.propTypes = {
  /**
   * Array of items.
   */
  items: arrayOf(shape(ITEM_SHAPE)),

  /**
   * Called when an item is selected.
   * @param {SyntheticEvent} event - React's original SyntheticEvent.
   * @param {Item} item - Selected item.
   * @returns {void}
   */
  onItemSelect: func,

  /**
   * Called when the search input change.
   * @param {SyntheticEvent} event - React's original SyntheticEvent.
   * @param {string} query - Search query.
   * @returns {void}
   */
  onSearch: func,

  /**
   * Called when an open event happens.
   * @param {SyntheticEvent} event - React's original SyntheticEvent.
   * @returns {void}
   */
  onOpen: func,

  /**
   * Called when a close event happens.
   * @param {SyntheticEvent} event - React's original SyntheticEvent.
   * @returns {void}
   */
  onClose: func,

  /**
   * Whether or not the dropdown should close when an item is selected.
   */
  closeOnSelect: bool,

  /**
   * Delay before initiating a search when the query change.
   */
  debounceDelay: number,

  /**
   * Message to display when there are no items matching the query.
   */
  noResultsMessage: string,

  /**
   * A React component that open the dropdown.
   */
  trigger: element,

  /**
   * The trigger text.
   */
  triggerText: string,

  /**
   * A dropdown trigger can have different sizes.
   */
  triggerSize: oneOf(SIZES),

  /**
   * A React component to display the items.
   */
  menu: element,

  /**
   * [Shorthand](/?path=/docs/getting-started-shorthand-props--page) for a [text input](/?path=/docs/components-textinput--default-story).
   */
  searchInput: oneOfType([element, object]),

  /**
   * The search input placeholder text.
   */
  placeholder: string,

  /**
   * A controlled open value that determined whether or not the dropdown is displayed.
   */
  open: bool,

  /**
   * @ignore
   */
  active: bool,

  /**
   * @ignore
   */
  focus: bool,

  /**
   * @ignore
   */
  hover: bool
};
TagsPickerDropdown.defaultProps = {
  debounceDelay: 200,
  menu: /*#__PURE__*/React.createElement(TagsPickerDropdownMenu, null),
  trigger: /*#__PURE__*/React.createElement(TagsPickerDropdownTrigger, null),
  searchInput: /*#__PURE__*/React.createElement(TagsPickerDropdownSearchInput, null)
};